import pandas as pd
from enums.DatabaseTypeEnum import DatabaseTypeEnum
from sqlalchemy.types import VARCHAR

import logging

logger = logging.getLogger()


def save_from_dataframe(df: pd.DataFrame, tablename: str, connection, is_first: bool = False) -> bool:
    save = False
    try:
        chunksize = 2000
        db_type = connection.db_type
        connection = connection.engine
        with connection.connect() as conn:
            if db_type == DatabaseTypeEnum.ORACLE.value:
                dtypes = {c: VARCHAR(df[c].str.len().max()) for c in df.columns[df.dtypes == 'object'].tolist()}
                df.to_sql(tablename, conn, if_exists='append', dtype=dtypes, index=False, chunksize=chunksize)
            else:
                df.to_sql(tablename, conn, if_exists='append', index=False, chunksize=chunksize)
            save = True
    except Exception as e:
        logger.error(f"Error guardando resultados desde dataframe. {e}")
        raise AssertionError(f"Error guardando resultados desde dataframe. {e}")
    finally:
        return save
