import logging
import time
from typing import List
import re

from enums.DatabaseTypeEnum import DatabaseTypeEnum

logger = logging.getLogger()


def execute_transformations(commands: List[str], engine):
    try:
        with engine.connect() as connection:
            for command in commands:
                logger.debug(f"Ejecutando comando de transformación: {command}")
                _ = connection.execute(command)
    except Exception as e:
        logger.error(f"Error ejecutando comando de transformación. {e}")


def delete_table(tablename: str, connection) -> bool:
    engine = connection.engine
    delete = False
    try:
        if connection.db_type == DatabaseTypeEnum.ORACLE.value:
            command = f'DROP TABLE {tablename}'
            start_time = time.time()
            with engine.connect() as conn:
                try:
                    check_query = f"SELECT table_name FROM all_tables WHERE table_name = '{tablename.upper()}'"
                    result = conn.execute(check_query)
                    exists = bool(result.fetchone())
                except Exception as e:
                    logger.error(f"Tabla no encontrada. {e}")
                if exists:
                    _ = conn.execute(command)
                    delete = True
                    logger.debug(f"Duración de borrado: {time.time() - start_time}")
        else:
            command = f'DROP TABLE IF EXISTS {tablename}'
            start_time = time.time()
            with engine.connect() as conn:
                try:
                    _ = conn.execute(command)
                    delete = True
                    logger.debug(f"Duración de borrado: {time.time() - start_time}")
                except Exception as e:
                    logger.error(f"Tabla no encontrada. {e}")
    except Exception as e:
        logger.error(f"Error borrando tabla {tablename}. {e}")
    finally:
        return delete


def delete_procedure(procedure: str, connection) -> bool:
    engine = connection.engine
    delete = False
    try:
        if connection.db_type == DatabaseTypeEnum.ORACLE.value:
            command = f"DROP PROCEDURE {procedure}"
            start_time = time.time()
            with engine.connect() as conn:
                try:
                    proc = re.match(r'([^(]+)', procedure).group(1)
                    check_query = f"SELECT object_name FROM all_procedures WHERE object_name = '{proc.upper()}'"
                    result = conn.execute(check_query)
                    exists = bool(result.fetchone())
                except Exception as e:
                    logger.error(f"Procedure no encontrado. {e}")
                if exists:
                    _ = conn.execute(command)
                    delete = True
                    logger.debug(f"Duración de borrado: {time.time() - start_time}")
        else:
            command = f'DROP PROCEDURE IF EXISTS {procedure}'
            start_time = time.time()
            with engine.connect() as conn:
                try:
                    _ = conn.execute(command)
                    delete = True
                    logger.debug(f"Duración de borrado: {time.time() - start_time}")
                except Exception as e:
                    logger.error(f"Procedure no encontrada. {e}")
    except Exception as e:
        logger.error(f"Error borrando procedure {procedure}. {e}")
    finally:
        return delete
