from airflow.utils.db import provide_session
from sqlalchemy import or_
from airflow.models import XCom, TaskInstance, Variable
import logging
from typing import List, Tuple

from enums.OperationTypeEnum import OperationTypeEnum

logger = logging.getLogger()


def save_commands_to_xcom(dataset: List[Tuple[str, List[str]]], task, extract_mask: str, transform_mask: str,
                          procedure_mask: str, order_delimiter: str) -> None:
    final_names_xcom = []
    try:
        for data in dataset:
            logger.info(f"Guardando Xcom en llave {data[0]}")
            name = data[0]
            base_name = name
            if order_delimiter == ".":
                base_name = base_name[:base_name.rfind(".")]
            order = base_name.split(order_delimiter)
            if len(order) < 2:
                raise AssertionError(f"Script {name} no tiene prefijo de orden. Validar nombre de script")
            name += "_" + order[0]
            if name.find(extract_mask) != -1:
                name = OperationTypeEnum.SELECT.value + "|" + name
            elif name.find(transform_mask) != -1:
                name = OperationTypeEnum.TRANSFORM.value + "|" + name
            elif name.find(procedure_mask) != -1:
                name = OperationTypeEnum.PROCEDURE.value + "|" + name
            task.xcom_push(key=name, value=data[1])
            final_names_xcom.append(name)
        task.xcom_push(key="XCOM-EXTRACTION-NAMES", value=final_names_xcom)
    except Exception as e:
        raise AssertionError(f"Error guardando comandos en variables en xcom. {e}")


def save_values_to_xcom(value: str, task, task_name: str, key: str) -> None:
    final_names_xcom = []
    try:
        xcom_names = task.xcom_pull(task_ids=task_name, key=key)
        if not xcom_names:
            final_names_xcom.append(value)
        else:
            final_names_xcom = xcom_names
            final_names_xcom.append(value)
        task.xcom_push(key=key, value=final_names_xcom)
        logger.info(f"Guardado correctamente los nombres de las tablas")
    except Exception as e:
        logger.error(f"Error guardando nombres en variables xcom. {e}")


def delete_all_xcom_tasks() -> None:
    try:
        @provide_session
        def cleanup_xcom(session=None):
            session.query(XCom).filter(or_(XCom.task_id == "SCRIPTS-EXTRACTOR", XCom.task_id == "EXTRACTORS",
                                           XCom.task_id == "GENERATORS", XCom.task_id == "TRANSFORMATIONS",
                                           XCom.task_id == "VALIDATE_EXTRACTION", XCom.task_id == "VALIDATE_GENERATOR",
                                           XCom.task_id == "VALIDATE_TRANSFORMATION", XCom.task_id == "CONTROL-EXTRACTOR")).delete()
            session.query(Variable).filter(or_(Variable.key == "SELECTS", Variable.key == "TRANSFORMS",
                                               Variable.key == "GENERATES", Variable.key == "CLEANS")).delete()
            delete_task_instances()
            session.query(XCom).filter(XCom.dag_id == "BCOM_DAG_TRANSFORMACIONES4").delete()
        cleanup_xcom()
    except Exception as e:
        logger.error(f"Error borrando todas las variables xcom del DAG actual. {e}")


def delete_task_instances() -> None:
    try:
        @provide_session
        def cleanup_taskinstances(session=None):
            session.query(TaskInstance).filter(TaskInstance.state == "removed").delete()
        cleanup_taskinstances()
    except Exception as e:
        logger.error(f"Error borrando TaskInstances. {e}")
